/*
 
Copyright (C) 2006 NTT DATA Corporation
 
This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation, version 2.
 
This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied 
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.
 
*/

package com.clustercontrol.monitor.factory;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.naming.NamingException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.clustercontrol.bean.ConfirmConstant;
import com.clustercontrol.bean.FacilityTargetConstant;
import com.clustercontrol.bean.PriorityConstant;
import com.clustercontrol.bean.Property;
import com.clustercontrol.bean.ViewListInfo;
import com.clustercontrol.monitor.bean.ReportEventInfo;
import com.clustercontrol.monitor.ejb.entity.EventLogLocal;
import com.clustercontrol.monitor.ejb.entity.EventLogUtil;
import com.clustercontrol.monitor.ejb.session.MonitorControllerBean;
import com.clustercontrol.repository.ejb.session.RepositoryControllerBean;
import com.clustercontrol.repository.ejb.session.RepositoryControllerUtil;
import com.clustercontrol.util.PropertyUtil;
import com.clustercontrol.util.apllog.AplLogger;

/**
 * イベント情報を検索するクラスです。
 * 
 * @version 2.1.0
 * @since 1.0.0
 */
public class SelectEvent {
	
	/** ログ出力のインスタンス。 */
	protected static Log m_log = LogFactory.getLog( SelectEvent.class );
	
	/** 
	 * 表示イベント数（デフォルト値）。<BR>
	 * 監視[イベント]ビューに表示するイベント表示数を格納します。
	 */
	private final static int MAX_DISPLAY_NUMBER = 500;
	
	/**
	 * イベント一覧情報を返します。<BR>
	 * 表示イベント数を越えた場合は、表示イベント数分のイベント情報一覧を返します。
	 * <p>
	 * <ol>
	 * <li>引数で指定されたファシリティ配下のファシリティを、指定されたファシリティのターゲットで取得します。</li>
	 * <li>取得したファシリティに属するイベント情報を取得します。</li>
	 * <li>１イベント情報をテーブルのカラム順（{@link com.clustercontrol.monitor.bean.EventTabelDefine}）に、リスト（{@link ArrayList}）にセットします。</li>
	 * <li>この１イベント情報を保持するリストを、イベント情報一覧を保持するリスト（{@link ArrayList}）に格納します。<BR>
	 *  <dl>
	 *  <dt>イベント情報一覧（Objectの2次元配列）</dt>
	 *  <dd>{ イベント情報1 {カラム1の値, カラム2の値, … }, イベント情報2{カラム1の値, カラム2の値, …}, … }</dd>
	 *  </dl>
	 * </li>
	 * <li>イベント情報一覧，全イベント数 および 全重要度別イベント数を、ビュー一覧情報（{@link com.clustercontrol.bean.ViewListInfo}）にセットし返します。</li>
	 * </ol>
	 * 
	 * @param facilityId 取得対象の親ファシリティID
	 * @param level 取得対象のファシリティのターゲット（配下全て／直下のみ）
	 * @param messages 表示イベント数
	 * @return ビュー一覧情報
	 * @throws CreateException
	 * @throws FinderException
	 * @throws NamingException
	 * @throws SQLException
	 * 
	 * @see com.clustercontrol.repository.ejb.session.RepositoryControllerBean#ALL
	 * @see com.clustercontrol.repository.ejb.session.RepositoryControllerBean#ONE_LEVEL
	 * @see com.clustercontrol.repository.ejb.session.RepositoryControllerBean#getFacilityIdList(String, int)
	 * @see com.clustercontrol.monitor.ejb.entity.EventLogBean#ejbFindEvent(String[], Integer, Timestamp, Timestamp, Timestamp, Timestamp, String, String, Integer, boolean, Integer)
	 * @see com.clustercontrol.monitor.ejb.entity.EventLogBean#ejbHomeCountEvent(String[], Integer, Timestamp, Timestamp, Timestamp, Timestamp, String, String, Integer)
	 * @see com.clustercontrol.monitor.bean.EventTabelDefine
	 */
	public ViewListInfo getEventList(String facilityId, int level, int messages)
			throws CreateException, FinderException, NamingException, SQLException {

		ViewListInfo list = new ViewListInfo();
		try {
			String[] facilityIds = null;

			// ファシリティが指定されない（最上位）場合は、ファシリティIDを指定せずに検索を行う
			if (facilityId != null && !"".equals(facilityId)) {

				if (level == MonitorControllerBean.ONLY) {
					if (facilityId != null && !"".equals(facilityId)) {
						facilityIds = new String[1];
						facilityIds[0] = facilityId;
					} else {
						return null;
					}
				} else {
					// 直下 または 配下すべてのファシリティIDを取得
					ArrayList facilityIdList = 
						RepositoryControllerUtil.getLocalHome().create().getFacilityIdList(
									facilityId, level);

					if (facilityIdList != null && facilityIdList.size() > 0) {
						// スコープの場合
						facilityIds = new String[facilityIdList.size()];
						facilityIdList.toArray(facilityIds);
					} else {
						if(facilityId != null){
							// ノードの場合
							facilityIds = new String[1];
							facilityIds[0] = facilityId;
						}
						else{
							// リポジトリが1件も登録されていない場合
							return null;
						}
					}
				}
			}
			
			if(messages <= 0){
				messages = MAX_DISPLAY_NUMBER;
			}
			Integer limit = new Integer(messages + 1);

			// イベントログ情報一覧を取得
			Collection ct = EventLogUtil.getLocalHome().findEvent(
					facilityIds, 
					null, 
					null, 
					null, 
					null, 
					null, 
					null, 
					null, 
					new Integer(ConfirmConstant.TYPE_UNCONFIRMED),
					false,
					limit);

			//重要度別イベント数をカウント
			if(ct.size() > messages){
				//最大表示件数より大きい場合
				int critical = EventLogUtil.getLocalHome().countEvent(
						facilityIds, 
						PriorityConstant.TYPE_CRITICAL, 
						null, 
						null, 
						null, 
						null, 
						null, 
						null, 
						new Integer(ConfirmConstant.TYPE_UNCONFIRMED));
				
				int warning = EventLogUtil.getLocalHome().countEvent(
						facilityIds, 
						PriorityConstant.TYPE_WARNING, 
						null, 
						null, 
						null, 
						null, 
						null, 
						null, 
						new Integer(ConfirmConstant.TYPE_UNCONFIRMED));
				
				int info = EventLogUtil.getLocalHome().countEvent(
						facilityIds, 
						PriorityConstant.TYPE_INFO, 
						null, 
						null, 
						null, 
						null, 
						null, 
						null, 
						new Integer(ConfirmConstant.TYPE_UNCONFIRMED));
				
				int unKnown = EventLogUtil.getLocalHome().countEvent(
						facilityIds, 
						PriorityConstant.TYPE_UNKNOWN, 
						null, 
						null, 
						null, 
						null, 
						null, 
						null, 
						new Integer(ConfirmConstant.TYPE_UNCONFIRMED));
				
				int total = critical + warning + info + unKnown;
				
				list.setCritical(critical);
				list.setWarning(warning);
				list.setInfo(info);
				list.setUnKnown(unKnown);
				list.setTotal(total);
			}
			
			// 2次元配列に変換
			this.collectionToEventList(ct, list, messages);

		} catch (CreateException e) {
			outputLog(e, "getEventList()", facilityId, null, null, null, null, null, null, null, null, null);
			throw e;
		} catch (FinderException e) {
			outputLog(e, "getEventList()", facilityId, null, null, null, null, null, null, null, null, null);
			throw e;
		} catch (NamingException e) {
			outputLog(e, "getEventList()", facilityId, null, null, null, null, null, null, null, null, null);
			throw e;
		}
		
		return list;
	}

	/**
	 * 引数で指定された条件に一致するイベント一覧情報を返します。<BR>
	 * 表示イベント数を越えた場合は、表示イベント数分のイベント情報一覧を返します。
	 * <p>
	 * <ol>
	 * <li>引数で指定されたプロパティに格納された検索条件を、プロパティユーティリティ（{@link com.clustercontrol.util.PropertyUtil}）を使用して取得します。</li>
	 * <li>引数で指定されたファシリティ配下のファシリティと検索条件を基に、イベント情報を取得します。</li>
	 * <li>１イベント情報をテーブルのカラム順（{@link com.clustercontrol.monitor.bean.EventTabelDefine}）に、リスト（{@link ArrayList}）にセットします。</li>
	 * <li>この１イベント情報を保持するリストを、イベント情報一覧を保持するリスト（{@link ArrayList}）に格納します。<BR>
	 *  <dl>
	 *  <dt>イベント情報一覧（Objectの2次元配列）</dt>
	 *  <dd>{ イベント情報1{カラム1の値, カラム2の値, … }, イベント情報2{カラム1の値, カラム2の値, …}, … }</dd>
	 *  </dl>
	 * </li>
	 * <li>イベント情報一覧，全イベント数 および 全重要度別イベント数を、ビュー一覧情報（{@link com.clustercontrol.bean.ViewListInfo}）にセットし返します。</li>
	 * </ol>
	 * 
	 * @param facilityId 取得対象の親ファシリティID
	 * @param property 検索条件
	 * @param messages 表示イベント数
	 * @return ビュー一覧情報
	 * @throws CreateException
	 * @throws FinderException
	 * @throws NamingException
	 * @throws SQLException
	 * 
	 * @see com.clustercontrol.util.PropertyUtil#getPropertyValue(com.clustercontrol.bean.Property, java.lang.String)
	 * @see com.clustercontrol.repository.ejb.session.RepositoryControllerBean#getFacilityIdList(String, int)
	 * @see com.clustercontrol.monitor.ejb.entity.EventLogBean#ejbFindEvent(String[], Integer, Timestamp, Timestamp, Timestamp, Timestamp, String, String, Integer, boolean, Integer)
	 * @see com.clustercontrol.monitor.ejb.entity.EventLogBean#ejbHomeCountEvent(String[], Integer, Timestamp, Timestamp, Timestamp, Timestamp, String, String, Integer)
	 * @see com.clustercontrol.monitor.bean.EventTabelDefine
	 */
	public ViewListInfo getEventList(String facilityId, Property property, int messages)
			throws CreateException, FinderException, NamingException, SQLException {

		ViewListInfo list = new ViewListInfo();
		
		Integer priority = null;
		Timestamp outputFromDate = null;
		Timestamp outputToDate = null;
		Timestamp generationFromDate = null;
		Timestamp generationToDate = null;
		String facilityType = null;
		String application = null;
		String message = null;
		Integer confirmFlg = null;
		
		try {
			ArrayList values = null;

			//重要度取得
			values = PropertyUtil.getPropertyValue(property,
					StatusFilterProperty.PRIORITY);
			if (!"".equals((String) values.get(0))) {
				int priorityType = PriorityConstant
						.stringToType((String) values.get(0));
				if (priorityType != -1) {
					priority = new Integer(priorityType);
				}
			}

			//更新日時（自）取得
			values = PropertyUtil.getPropertyValue(property,
					EventFilterProperty.OUTPUT_FROM_DATE);
			if (values.get(0) instanceof Date) {
				outputFromDate = new Timestamp(((Date) values.get(0)).getTime());
				outputFromDate.setNanos(0);
			}

			//更新日時（至）取得
			values = PropertyUtil.getPropertyValue(property,
					EventFilterProperty.OUTPUT_TO_DATE);
			if (values.get(0) instanceof Date) {
				outputToDate = new Timestamp(((Date) values.get(0)).getTime());
				outputToDate.setNanos(999999999);
			}

			//出力日時（自）取得
			values = PropertyUtil.getPropertyValue(property,
					EventFilterProperty.GENERATION_FROM_DATE);
			if (values.get(0) instanceof Date) {
				generationFromDate = new Timestamp(((Date) values.get(0))
						.getTime());
				generationFromDate.setNanos(0);
			}

			//出力日時（至）取得
			values = PropertyUtil.getPropertyValue(property,
					EventFilterProperty.GENERATION_TO_DATE);
			if (values.get(0) instanceof Date) {
				generationToDate = new Timestamp(((Date) values.get(0))
						.getTime());
				generationToDate.setNanos(999999999);
			}

			//対象ファシリティ種別取得
			values = PropertyUtil.getPropertyValue(property,
					StatusFilterProperty.FACILITY_TYPE);
			if (!"".equals((String) values.get(0))) {
				facilityType = (String) values.get(0);
			}

			//アプリケーション取得
			values = PropertyUtil.getPropertyValue(property,
					StatusFilterProperty.APPLICATION);
			if (!"".equals((String) values.get(0))) {
				application = (String) values.get(0);
			}

			//メッセージ取得
			values = PropertyUtil.getPropertyValue(property,
					StatusFilterProperty.MESSAGE);
			if (!"".equals((String) values.get(0))) {
				message = (String) values.get(0);
			}

			// 確認有無取得
			values = PropertyUtil.getPropertyValue(property,
					EventFilterProperty.CONFIRMED);
			int confirmFlgType = ConfirmConstant.stringToType((String) values
					.get(0));
			if (confirmFlgType != -1) {
				confirmFlg = new Integer(confirmFlgType);
			}

			// 対象ファシリティのファシリティIDを取得
			// ファシリティが指定されない（最上位）場合は、ファシリティIDを指定せずに検索を行う
			String[] facilityIds = null;
			if (facilityId != null && !"".equals(facilityId)) {

				int level = RepositoryControllerBean.ALL;
				if (FacilityTargetConstant.STRING_BENEATH.equals(facilityType)) {
					level = RepositoryControllerBean.ONE_LEVEL;
				}

				ArrayList facilityIdList = RepositoryControllerUtil
						.getLocalHome().create().getFacilityIdList(facilityId,
								level);

				if (facilityIdList != null && facilityIdList.size() > 0) {
					// スコープの場合
					facilityIds = new String[facilityIdList.size()];
					facilityIdList.toArray(facilityIds);
				}
				else {
					if(facilityId != null){
						// ノードの場合
						facilityIds = new String[1];
						facilityIds[0] = facilityId;
					}
					else{
						// リポジトリが1件も登録されていない場合
						return null;
					}
				}
			}

			if(messages <= 0){
				messages = MAX_DISPLAY_NUMBER;
			}
			Integer limit = new Integer(messages + 1);
			
			// イベントログ情報一覧を取得
			Collection ct = EventLogUtil.getLocalHome().findEvent(facilityIds, 
					priority, 
					outputFromDate, 
					outputToDate,
					generationFromDate, 
					generationToDate, 
					application, 
					message,
					confirmFlg,
					false,
					limit);
			
			//重要度別イベント数をカウント
			if(ct.size() > messages){
				//最大表示件数より大きい場合
				int critical = EventLogUtil.getLocalHome().countEvent(
						facilityIds, 
						PriorityConstant.TYPE_CRITICAL, 
						outputFromDate, 
						outputToDate,
						generationFromDate, 
						generationToDate, 
						application, 
						message,
						confirmFlg);
				
				int warning = EventLogUtil.getLocalHome().countEvent(
						facilityIds, 
						PriorityConstant.TYPE_WARNING, 
						outputFromDate, 
						outputToDate,
						generationFromDate, 
						generationToDate, 
						application, 
						message,
						confirmFlg);
				
				int info = EventLogUtil.getLocalHome().countEvent(
						facilityIds, 
						PriorityConstant.TYPE_INFO, 
						outputFromDate, 
						outputToDate,
						generationFromDate, 
						generationToDate, 
						application, 
						message,
						confirmFlg);
				
				int unKnown = EventLogUtil.getLocalHome().countEvent(
						facilityIds, 
						PriorityConstant.TYPE_UNKNOWN, 
						outputFromDate, 
						outputToDate,
						generationFromDate, 
						generationToDate, 
						application, 
						message,
						confirmFlg);

				//フィルタに重要度が設定されていた場合のカウント数を設定				
				if (priority != null) {
					switch (priority) {
					
					case PriorityConstant.TYPE_CRITICAL:
						warning = 0;
						info = 0;
						unKnown = 0;
						break;
						
					case PriorityConstant.TYPE_WARNING:
						critical = 0;
						info = 0;
						unKnown = 0;
						break;
						
					case PriorityConstant.TYPE_INFO:
						critical = 0;
						warning = 0;
						unKnown = 0;
						break;
						
					case PriorityConstant.TYPE_UNKNOWN:
						critical = 0;
						warning = 0;
						info = 0;
						break;
					}
				}
				
				int total = critical + warning + info + unKnown;
				
				list.setCritical(critical);
				list.setWarning(warning);
				list.setInfo(info);
				list.setUnKnown(unKnown);
				list.setTotal(total);
			}

			// 2次元配列に変換
			this.collectionToEventList(ct, list, messages);

		} catch (CreateException e) {
			outputLog(e, "getEventList()", facilityId, priority, outputFromDate, outputToDate, generationFromDate, generationToDate, facilityType, application, message, confirmFlg);
			throw e;
		} catch (FinderException e) {
			outputLog(e, "getEventList()", facilityId, priority, outputFromDate, outputToDate, generationFromDate, generationToDate, facilityType, application, message, confirmFlg);
			throw e;
		} catch (NamingException e) {
			outputLog(e, "getEventList()", facilityId, priority, outputFromDate, outputToDate, generationFromDate, generationToDate, facilityType, application, message, confirmFlg);
			throw e;
		}
		
		return list;
	}

	/**
	 * 重要度が最高で受信日時が最新のイベント情報を返します。
	 * <p>
	 * <ol>
	 * <li>引数で指定されたファシリティ配下のファシリティを、指定されたファシリティのターゲットで取得します。</li>
	 * <li>取得したファシリティに属する重要度が最高 かつ 受信日時が最新の未確認のイベント情報を取得し返します。</li>
	 * </ol>
	 * 
	 * @param facilityId 取得対象の親ファシリティID
	 * @param level 取得対象のファシリティのターゲット（配下全て／直下のみ）
	 * @return イベントのローカルコンポーネントインターフェース
	 * @throws CreateException
	 * @throws FinderException
	 * @throws NamingException
	 * 
	 * @see com.clustercontrol.repository.ejb.session.RepositoryControllerBean#ALL
	 * @see com.clustercontrol.repository.ejb.session.RepositoryControllerBean#ONE_LEVEL
	 * @see com.clustercontrol.repository.ejb.session.RepositoryControllerBean#getFacilityIdList(String, int)
	 * @see com.clustercontrol.monitor.ejb.entity.EventLogBean#ejbFindHighPriorityEvent(String[], Integer, Timestamp, Timestamp, Timestamp, Timestamp, String, String, Integer)
	 */
	public EventLogLocal getHighPriorityEvent(String facilityId, int level)
			throws CreateException, FinderException, NamingException {

		EventLogLocal event = null;
		try {
			String[] facilityIds = null;
			if (level == MonitorControllerBean.ONLY) {
				if (facilityId != null && !"".equals(facilityId)) {
					facilityIds = new String[1];
					facilityIds[0] = facilityId;
				} else {
					return null;
				}
			} else {
				// 直下 または 配下すべてのファシリティIDを取得
				ArrayList facilityIdList = RepositoryControllerUtil
						.getLocalHome().create().getFacilityIdList(facilityId,
								level);

				if (facilityIdList != null && facilityIdList.size() > 0) {
					// スコープの場合
					facilityIds = new String[facilityIdList.size()];
					facilityIdList.toArray(facilityIds);
				} else {
					if(facilityId != null){
						// ノードの場合
						facilityIds = new String[1];
						facilityIds[0] = facilityId;
					}
					else{
						// リポジトリが1件も登録されていない場合
						return null;
					}
				}
			}

//			// イベントログ情報一覧を取得
//			Collection ct = EventLogUtil.getLocalHome().findHighPriorityEvent(facilityIds, 
//					null, 
//					null, 
//					null, 
//					null, 
//					null, 
//					null, 
//					null, 
//					new Integer(ConfirmConstant.TYPE_UNCONFIRMED));
	
			// 重要度のリストを取得する
			int[] priorityList = PriorityConstant.PRIORITY_LIST;			
			for(int i=0; i<priorityList.length; i++){
				// イベントログ情報一覧から重要度が危険のもので、最近出力されたイベントを取得する。
				Collection ct = EventLogUtil.getLocalHome().findHighPriorityEvent(facilityIds, 
						priorityList[i], 
						null, 
						null, 
						null, 
						null, 
						null, 
						null, 
						new Integer(ConfirmConstant.TYPE_UNCONFIRMED));
				// 重要度の高いもの順にループされるため、取得できた場合は、それを返す。
				Iterator itr = ct.iterator();
				// イテレータで参照するが、0件か１件しかない。
				if (itr.hasNext()) {
					event = (EventLogLocal) itr.next();
					return event;
				}
			}

		} catch (CreateException e) {
			m_log.debug("getHighPriorityEvent():" + e.getMessage());
			throw e;
		} catch (FinderException e) {
			m_log.debug("getHighPriorityEvent():" + e.getMessage());
			throw e;
		} catch (NamingException e) {
			m_log.debug("getHighPriorityEvent():" + e.getMessage());
			throw e;
		}
		return event;
	}
	
	/**
	 * 引数で指定された条件に一致する帳票出力用のイベント情報一覧を返します。
	 * <p>
	 * <ol>
	 * <li>引数で指定されたプロパティに格納された検索条件を、プロパティユーティリティ（{@link com.clustercontrol.util.PropertyUtil}）を使用して取得します。</li>
	 * <li>引数で指定されたファシリティ配下のファシリティと検索条件を基に、イベント情報を取得します。</li>
	 * <li>１イベント情報を帳票出力用イベント情報（{@link com.clustercontrol.monitor.bean.ReportEventInfo}）にセットします。</li>
	 * <li>この帳票出力用イベント情報を、イベント情報一覧を保持するリスト（{@link ArrayList}）にセットし返します。<BR>
	 * </ol>
	 * 
	 * @param facilityId 取得対象の親ファシリティID
	 * @param property 検索条件
	 * @return 帳票出力用イベント情報一覧（{@link com.clustercontrol.monitor.bean.ReportEventInfo}のリスト）
	 * @throws CreateException
	 * @throws FinderException
	 * @throws NamingException
	 * @throws IndexOutOfBoundsException
	 * 
	 * @since 2.1.0
	 * 
	 * @see com.clustercontrol.util.PropertyUtil#getPropertyValue(com.clustercontrol.bean.Property, java.lang.String)
	 * @see com.clustercontrol.repository.ejb.session.RepositoryControllerBean#getFacilityIdList(String, int)
	 * @see com.clustercontrol.monitor.ejb.entity.EventLogBean#ejbFindEvent(String[], Integer, Timestamp, Timestamp, Timestamp, Timestamp, String, String, Integer, boolean, Integer)
	 * @see com.clustercontrol.monitor.bean.ReportEventInfo
	 */
	public ArrayList getEventListForReport(String facilityId, Property property)
			throws CreateException, FinderException, NamingException, IndexOutOfBoundsException {

		ArrayList list = null;
		
		Integer priority = null;
		Timestamp outputFromDate = null;
		Timestamp outputToDate = null;
		Timestamp generationFromDate = null;
		Timestamp generationToDate = null;
		String facilityType = null;
		String application = null;
		String message = null;
		Integer confirmFlg = null;
		
		try {
			ArrayList values = null;

			//重要度取得
			values = PropertyUtil.getPropertyValue(property,
					StatusFilterProperty.PRIORITY);
			if (!"".equals((String) values.get(0))) {
				int priorityType = PriorityConstant
						.stringToType((String) values.get(0));
				if (priorityType != -1) {
					priority = new Integer(priorityType);
				}
			}

			//更新日時（自）取得
			values = PropertyUtil.getPropertyValue(property,
					EventFilterProperty.OUTPUT_FROM_DATE);
			if (values.get(0) instanceof Date) {
				outputFromDate = new Timestamp(((Date) values.get(0)).getTime());
				outputFromDate.setNanos(0);
			}

			//更新日時（至）取得
			values = PropertyUtil.getPropertyValue(property,
					EventFilterProperty.OUTPUT_TO_DATE);
			if (values.get(0) instanceof Date) {
				outputToDate = new Timestamp(((Date) values.get(0)).getTime());
				outputToDate.setNanos(999999999);
			}

			//出力日時（自）取得
			values = PropertyUtil.getPropertyValue(property,
					EventFilterProperty.GENERATION_FROM_DATE);
			if (values.get(0) instanceof Date) {
				generationFromDate = new Timestamp(((Date) values.get(0))
						.getTime());
				generationFromDate.setNanos(0);
			}

			//出力日時（至）取得
			values = PropertyUtil.getPropertyValue(property,
					EventFilterProperty.GENERATION_TO_DATE);
			if (values.get(0) instanceof Date) {
				generationToDate = new Timestamp(((Date) values.get(0))
						.getTime());
				generationToDate.setNanos(999999999);
			}

			//対象ファシリティ種別取得
			values = PropertyUtil.getPropertyValue(property,
					StatusFilterProperty.FACILITY_TYPE);
			if (!"".equals((String) values.get(0))) {
				facilityType = (String) values.get(0);
			}

			//アプリケーション取得
			values = PropertyUtil.getPropertyValue(property,
					StatusFilterProperty.APPLICATION);
			if (!"".equals((String) values.get(0))) {
				application = (String) values.get(0);
			}

			//メッセージ取得
			values = PropertyUtil.getPropertyValue(property,
					StatusFilterProperty.MESSAGE);
			if (!"".equals((String) values.get(0))) {
				message = (String) values.get(0);
			}

			// 確認有無取得
			values = PropertyUtil.getPropertyValue(property,
					EventFilterProperty.CONFIRMED);
			int confirmFlgType = ConfirmConstant.stringToType((String) values
					.get(0));
			if (confirmFlgType != -1) {
				confirmFlg = new Integer(confirmFlgType);
			}

			// 対象ファシリティのファシリティIDを取得
			// ファシリティが指定されない（最上位）場合は、ファシリティIDを指定せずに検索を行う
			String[] facilityIds = null;
			if (facilityId != null && !"".equals(facilityId)) {

				int level = RepositoryControllerBean.ALL;
				if (FacilityTargetConstant.STRING_BENEATH.equals(facilityType)) {
					level = RepositoryControllerBean.ONE_LEVEL;
				}

				ArrayList facilityIdList = RepositoryControllerUtil
						.getLocalHome().create().getFacilityIdList(facilityId,
								level);

				if (facilityIdList != null && facilityIdList.size() > 0) {
					// スコープの場合
					facilityIds = new String[facilityIdList.size()];
					facilityIdList.toArray(facilityIds);
				}
				else {
					if(facilityId != null){
						// ノードの場合
						facilityIds = new String[1];
						facilityIds[0] = facilityId;
					}
					else{
						// リポジトリが1件も登録されていない場合
						return null;
					}
				}
			}

			// イベントログ情報一覧を取得
			Collection ct = EventLogUtil.getLocalHome().findEvent(facilityIds, 
					priority, 
					outputFromDate, 
					outputToDate,
					generationFromDate, 
					generationToDate, 
					application, 
					message,
					confirmFlg,
					true,
					null);
			
//			// 取得したイベントが最大表示件数より大きい場合、エラーを発生させる
//			if(m_eventMaxDisplayNumber != -1 && ct.size() > m_eventMaxDisplayNumber){
//				throw new java.lang.IndexOutOfBoundsException();
//			}

			// 帳票出力用に変換
			list = this.collectionToArrayForReportd(ct);

		} catch (CreateException e) {
			outputLog(e, "getEventListForReport()", facilityId, priority, outputFromDate, outputToDate, generationFromDate, generationToDate, facilityType, application, message, confirmFlg);
			throw e;
		} catch (FinderException e) {
			outputLog(e, "getEventListForReport()", facilityId, priority, outputFromDate, outputToDate, generationFromDate, generationToDate, facilityType, application, message, confirmFlg);
			throw e;
		} catch (NamingException e) {
			outputLog(e, "getEventListForReport()", facilityId, priority, outputFromDate, outputToDate, generationFromDate, generationToDate, facilityType, application, message, confirmFlg);
			throw e;
		}
		return list;
	}

	/**
	 * DBより取得したイベント情報をイベント一覧情報に格納します。
	 * <p>
	 * <ol>
 	 * <li>１イベント情報をテーブルのカラム順（{@link com.clustercontrol.monitor.bean.EventTabelDefine}）に、リスト（{@link ArrayList}）にセットします。</li>
	 * <li>この１イベント情報を保持するリストを、イベント情報一覧を保持するリスト（{@link ArrayList}）に格納します。<BR>
	 *  <dl>
	 *  <dt>イベント情報一覧（Objectの2次元配列）</dt>
	 *  <dd>{ イベント情報1 {カラム1の値, カラム2の値, … }, イベント情報2{カラム1の値, カラム2の値, …}, … }</dd>
	 *  </dl>
	 * </li>
	 * <li>イベント情報一覧を、引数で指定されたビュー一覧情報（{@link com.clustercontrol.bean.ViewListInfo}）にセットします。</li>
	 * </ol>
	 * <p>
	 * また、イベント数を重要度毎にカウントし、
	 * 表示イベント数よりもイベント数が少ない場合は、重要度別イベント数を引数で指定されたビュー一覧情報にセットします。
	 * 
	 * @param ct イベント情報取得結果
	 * @param eventList ビュー一覧情報
	 * @param messages イベント最大表示件数
	 * 
	 * @see com.clustercontrol.monitor.bean.EventTabelDefine
	 */
	@SuppressWarnings("unchecked")
	private void collectionToEventList(Collection ct, ViewListInfo eventList, int messages) {
		int critical = 0;
	    int warning = 0;
	    int info = 0;
	    int unKnown = 0;
	    int total = ct.size();
		
		ArrayList list = new ArrayList();
		Iterator itr = ct.iterator();
		while (itr.hasNext()) {
			EventLogLocal event = (EventLogLocal) itr.next();

			ArrayList eventInfo = new ArrayList();
			eventInfo.add(event.getPriority());
			eventInfo.add(new Date(event.getOutputDate().getTime()));
			eventInfo.add(new Date(event.getGenerationDate().getTime()));
			eventInfo.add(event.getPluginId());
			eventInfo.add(event.getMonitorId());
			eventInfo.add(event.getFacilityId());
			eventInfo.add(event.getScopeText());
			eventInfo.add(event.getApplication());
			eventInfo.add(event.getMessageId());
			eventInfo.add(event.getMessage());
			eventInfo.add(event.getConfirmFlg());
			list.add(eventInfo);

			//重要度別イベント数をカウント
			if(ct.size() <= messages){
				//最大表示件数以下の場合

				if(event.getPriority().intValue() == PriorityConstant.TYPE_CRITICAL)
					critical++;
				else if(event.getPriority().intValue() == PriorityConstant.TYPE_WARNING)
					warning++;
				else if(event.getPriority().intValue() == PriorityConstant.TYPE_INFO)
					info++;
				else if(event.getPriority().intValue() == PriorityConstant.TYPE_UNKNOWN)
					unKnown++;
			}
			
			//取得したイベントを最大表示件数まで格納したら終了
			if(list.size() >= messages)
				break;
		}
		
		//イベント数を設定
		if(ct.size() <= messages){
			eventList.setCritical(critical);
			eventList.setWarning(warning);
			eventList.setInfo(info);
			eventList.setUnKnown(unKnown);
			eventList.setTotal(total);
		}
		
		eventList.setList(list);
	}
	
	/**
	 * DBより取得したイベント情報を帳票出力用イベント情報一覧に格納します。
	 * 
	 * @param ct イベント情報取得結果
	 * @return 帳票出力用イベント情報一覧
	 * 
	 * @version 2.1.0
	 * @since 2.1.0
	 */
	private ArrayList<ReportEventInfo> collectionToArrayForReportd(Collection ct) {

		ArrayList<ReportEventInfo> list = new ArrayList<ReportEventInfo>();
		Iterator itr = ct.iterator();
		while (itr.hasNext()) {
			EventLogLocal event = (EventLogLocal) itr.next();

			ReportEventInfo info = new ReportEventInfo();
			
			info.setApplication(event.getApplication());
			info.setConfirmFlg(ConfirmConstant.typeToString(event.getConfirmFlg().intValue()));
			info.setFacilityId(event.getFacilityId());
			info.setGenerationDate(new Date(event.getGenerationDate().getTime()));
			info.setMessage(event.getMessage());
			info.setMessageId(event.getMessageId());
			info.setMessageOrg(event.getMessageOrg());
			info.setMonitorId(event.getMonitorId());
			info.setOutputDate(new Date(event.getOutputDate().getTime()));
			info.setPluginId(event.getPluginId());
			info.setPriority(PriorityConstant.typeToString(event.getPriority().intValue()));
			info.setScopeText(event.getScopeText());
			
			list.add(info);
		}
		return list;
	}
	
	/**
     * アプリケーションログにログを出力
     * 
     * @param e 例外
     */
	private void outputLog(
			Exception e, 
			String method,
			String facilityId,
			Integer priority,
			Timestamp outputFromDate,
			Timestamp outputToDate,
			Timestamp generationFromDate,
			Timestamp generationToDate,
			String facilityType,
			String application,
			String message,
			Integer confirmFlg){
		
		int priorityInt = -1;
		String outputFromDateString = null;
		String outputToDateString = null;
		String generationFromDateString = null;
		String generationToDateString = null;
		int confirmFlgInt = -1;
		
		if(priority != null)
			priorityInt = priority.intValue();
		if(outputFromDate != null)
			outputFromDateString = outputFromDate.toString();
		if(outputToDate != null)
			outputToDateString = outputToDate.toString();
		if(generationFromDate != null)
			generationFromDateString = generationFromDate.toString();
		if(generationToDate != null)
			generationToDateString = generationToDate.toString();
		if(confirmFlg != null)
			confirmFlgInt = confirmFlg.intValue();
		
        AplLogger apllog = new AplLogger("MON", "mon");
        String[] args = {facilityId,PriorityConstant.typeToString(priorityInt),outputFromDateString,outputToDateString,generationFromDateString,generationToDateString,facilityType,application,message,ConfirmConstant.typeToString(confirmFlgInt)};
        apllog.put("SYS", "003", args);
		m_log.debug(method + ":" + e.getMessage());
	}
}
