/*
 
Copyright (C) 2006 NTT DATA Corporation
 
This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation, version 2.
 
This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied 
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.
 
*/

package com.clustercontrol.syslogng.forward;

import java.util.Properties;
import java.util.Timer;
import java.util.TimerTask;

import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;
import javax.naming.InitialContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.clustercontrol.repository.bean.TopicConstant;
import com.clustercontrol.repository.message.UpdateRepositoryInfo;
import com.clustercontrol.syslogng.message.UpdateMonitorRuleInfo;

/**
 * トピックを受信するクラスです。
 * トピックへの接続と、メッセージの受信を行います。
 * 
 * @version 2.0.0
 * @since 2.0.0
 */
public class UpdateRepositoryInfoReceiveTopic implements MessageListener, ExceptionListener {
	
	/** ログ出力のインスタンス。 */
	private static Log m_log = LogFactory.getLog(UpdateRepositoryInfoReceiveTopic.class);
	
	/** コネクションファクトリー。 */
	private static final String TOPIC_CON_FACTORY = "ConnectionFactory";
	
	/** 再接続間隔（ミリ秒）。 */
	private static final long RETRY_INTERVAL = 10000;
	
	/** 多重化ID。 */
	private final static String MULTI_ID = "multi.id";
	
	/** コネクションファクトリー。 */
	private TopicConnectionFactory m_factory;
	/** コネクション。 */
	protected TopicConnection m_con;
	/** トピック。 */
	protected Topic m_topic;
	/** セッション。 */
	protected TopicSession m_session;
	/** サブスクライバ。 */
	protected TopicSubscriber m_subscriber;
	
	/** EJBアクセスのインスタンス。 */
	protected EJBContoroler m_ejbContoroler;
	/** プロパティファイル情報。 */
	private Properties m_props;
	
	/** エラーフラグ。 */
	private boolean m_errFlg = false;
	
	/** メッセージサービス再接続用タイマー。*/
	protected Timer m_timer = new Timer(true);
	
	/**
	 * コンストラクタ。<BR>
	 * 初期処理を呼び出します。
	 * 
	 * @param ejbContoroler EJBアクセス
	 * @param props　プロパティファイル情報
	 * 
	 * @see #initial()
	 */
	@SuppressWarnings("unchecked")
	public UpdateRepositoryInfoReceiveTopic(EJBContoroler ejbContoroler, Properties props) {
		super();
		 
		m_props = props;
		m_ejbContoroler = ejbContoroler;
		 
		//接続処理
		initial();
	}
	
	/** 
	 * 引数で指定された受信メッセージを元に、キャッシュの更新を行います。<BR>
	 * 特定のメッセージを受信した場合のみ処理を行います。受信メッセージは下記の通りです。
	 * 
	 * <p><li>{@link com.clustercontrol.repository.message.UpdateRepositoryInfo} : リポジトリ情報を更新します。
	 * <p><li>{@link com.clustercontrol.syslogng.message.UpdateMonitorRuleInfo} : フィルタ情報を更新します。
	 * 
	 * @see javax.jms.MessageListener#onMessage(javax.jms.Message)
	 * 
	 * @see com.clustercontrol.syslogng.forward.EJBContoroler#loadFilterInfo()
	 */
	public void onMessage(Message message) {
		
		if(message instanceof ObjectMessage){
			ObjectMessage objectMessage = (ObjectMessage)message;
			Object obj;
			try {
				obj = objectMessage.getObject();
			} catch (JMSException e) {
				return;
			}
			
			// リポジトリ更新時
			if(obj instanceof UpdateRepositoryInfo){
				m_log.debug("onMessage : Receive UpdateRepositoryInfo");
				
				m_ejbContoroler.loadFilterInfo();
			
			}
			// フィルタ条件更新時
			else if(obj instanceof UpdateMonitorRuleInfo){
				m_log.debug("onMessage : Receive UpdateMonitorRuleInfo");
				
				m_ejbContoroler.loadFilterInfo();
			
			}
			else{
				return;
			}
		}else{
			return;
		}
	}
	
	/**
	 * 通信エラーハンドラです。
	 * 
	 * @see javax.jms.ExceptionListener#onException(javax.jms.JMSException)
	 * 
	 * @see #setErrFlg(boolean) 
	 */
	public void onException(JMSException arg0) {
		setErrFlg(true);
	}
	
	/**
	 * エラーフラグを設定します。<BR>
	 * EJB再接続タイマータスクを登録します。
	 * 
	 * @param errFlg エラーの場合、<code> true </code>
	 */
	synchronized private void setErrFlg(boolean errFlg) {
		if (m_errFlg == false && errFlg == true) {
			
			m_timer.schedule(new ReSetupTask(), RETRY_INTERVAL,
					RETRY_INTERVAL);
			
		}
		m_errFlg = errFlg;
	}

	/**
	 * 再接続処理を行います。
	 * <p>
	 * <ol>
	 *  <li>終了処理を行います。</li>
	 *  <li>初期処理を行います。</li>
	 *  <li>エラーフラグを解除します。</li>
	 * </ol>
	 * 
	 * @see #terminate()
	 * @see #initial()
	 * @see #setErrFlg(boolean)
	 */
	synchronized private boolean reInitial() {
		boolean ret = false;
		
		terminate();
		
		if (initial()) {
			
			ret = true;
			
			//エラーフラグ解除
			setErrFlg(false);
			
		} else {
			
		}
		
		return ret;
	}
	
	/**
	 * JMS接続の終了処理を行います。
	 * <p>
	 * <ol>
	 * <li>セッションをクローズします。</li>
	 * <li>コネクションをクローズします。</li>
	 * </ol>
	 */
	public void terminate() {
		
		terminateSumscriber();
		
		
		try {
			if (m_session != null)
				m_session.close();
		} catch (JMSException e) {
		}
		
		try {
			if (m_con != null)
				m_con.close();
		} catch (JMSException e1) {
		}
	}
	
	/**
	 * トピック受信の終了処理を行います。
	 */
	private void terminateSumscriber() {
		try {
			if (m_subscriber != null)
				m_subscriber.close();
		} catch (JMSException e) {
		}
	}
	
	
	
	/**
	 * JMS接続の初期処理を行います。
	 * <p>
	 * <ol>
	 *  <li>コネクションファクトリと宛先を取得します。</li>
	 *  <li>コネクションファクトリより、JMSプロバイダとのコネクションを生成します。</li>
	 *  <li>コネクションよりセッションを生成します。</li>
	 *  <li>トピック接続を開始します。</li>
	 * </ol>
	 * 
	 * @see #initialTopic()
	 * 
	 * @return 成功した場合、<code> true </code>
	 */
	private boolean initial() {
		
		InitialContext con = null;
		
		try {
			//InitialContextの生成
			con = new InitialContext(m_props);

			//コネクションファクトリ生成
			m_factory = (TopicConnectionFactory)con.lookup(TOPIC_CON_FACTORY);
			
			//コネクション生成
			m_con = m_factory.createTopicConnection();
			
			//セッション生成
			m_session = m_con.createTopicSession(false,
					Session.AUTO_ACKNOWLEDGE);
			
			//メッセージTopic取得
			m_topic = (Topic)con.lookup(TopicConstant.TOPIC_NAME_EXECUTE);
			
			
			//エラーハンドらセット
			m_con.setExceptionListener(this);
			
			m_con.start();
			
			//トピック接続開始
			initialTopic();
			
			
		} catch (Exception e) {
			setErrFlg(true);
			return false;
		} finally {
			try {
				if (con != null)
					con.close();
			} catch (Exception e1) {
			}
		}
		return true;
		
	}
	
	/**
	 * トピック受信の初期処理を行います。
	 * 
	 * @return 成功した場合、<code> true </code>
	 */
	private boolean initialTopic() {
		
		
		//現在のTopic受信を終了
		terminateSumscriber();
		
		//新しいファシリティIDでトピック受信開始
		try {
	        //ファシリティIDでメッセージセレクターを作成
	        StringBuffer msgSelector = new StringBuffer();
	        
	        String multiId = m_props.getProperty(MULTI_ID, null);
	        
			if( multiId != null && multiId.length() > 0 ){
				msgSelector.append("MultiId='");
				msgSelector.append(multiId);
				msgSelector.append("' OR All='true'");
			}
	        
	        if(msgSelector.length() != 0){
		        m_subscriber = m_session.createSubscriber(m_topic, msgSelector.toString(), false);
		        
		        //コールバックを登録する
		        m_subscriber.setMessageListener(this);
	        }else{
				m_subscriber = m_session.createSubscriber(m_topic);
				
				//コールバックを登録する
				m_subscriber.setMessageListener(this);
	        }
		} catch (Exception e) {
			setErrFlg(true);
			return false;
		} finally {
		}
		return true;
		
	}
	/**
	 * EJB再接続タイマータスクです。<BR>
	 * 通信エラーとなった場合に定周期で呼ばれ、再接続を行います。
	 */
	protected class ReSetupTask extends TimerTask {
		
		/**
		 * 再接続を行い、成功した場合、このタスクをタイマーから削除します。
		 * 
		 * @see com.clustercontrol.syslogng.forward.UpdateRepositoryInfoReceiveTopic#reInitial()
		 */
		public void run() {
			if (reInitial()) {
				//このタスクをタイマーから解除
				cancel();
			}
		}
		
	}
	
}
